trap fo_close INT QUIT TERM EXIT

readonly FO_SCRIPT=$(basename "${0}")
readonly RED="\033[1;41m"
readonly NORM="\033[0m"
readonly UNMOUNTED="# "

auxilium_add nolock  -t opt -m "Don\'t create lock file"

another_instance() {
  logger -t "udev" "===> There is another instance running, exiting"
  exit 1
}

fo_repeat_char(){
  # repeat a char X 
	# param: char to repeat
	# param: nb of repeat
	local c="$1"
	local nb=$(seq 1 ${2})
	printf -- "${c}%.0s" ${nb}
}

fo_center_align(){
	# Center align a text
	# Param: Text to align
	# Param: len of final string
	# Param: If is disk to locate
	
	local lenTotal=$1
	local locate=$2
	[ -z "$3" ] && local text='Unknown' || local text=$3
	local lenText=${#text}
	let "nb_spaces =lenTotal - lenText"
	local space=' '
	local color=${NORM}
	if [ "$locate" == "True" ];then
		color=$RED
		[ -n "$AUXILIUM_ARG_redmine" ] && space='='
	fi
	let "first_nb = nb_spaces / 2"
	local first_spaces=$(fo_repeat_char "${space}" ${first_nb})
	let "end_nb = nb_spaces - first_nb"
	local end_spaces=$(fo_repeat_char "${space}" ${end_nb})
	local ret=$(printf -- "%s%s%s" "${first_spaces}" "$text" "${end_spaces}")
	let "lenTotal = lenTotal + 2"
	printf -- "${color}%${lenTotal}s${NORM}" "$ret"
}

fo_get_dev_letter(){
	# Print disk letter (a to z) by is index 
	# Param: Disk Index (beginning at 0)
	let "num = 97 + ${1}"
	printf \\$(printf '%03o' ${num})
}

fo_get_dev_ascii(){
	# Print disk ascii code of last letter of device name
	# Param: letter
	local name_id=$(printf "%d\n" \'${1:(-1)})
	let "ret = name_id - 97"
	echo $ret
}

fo_lock() {
	local LOCKDIR=/var/run/lock
	readonly FO_LOCKFILE=${LOCKDIR}/$(basename ${0})
	if [ -z "$AUXILIUM_ARG_nolock" ] ; then
		logger -t "udev" "===> Claming lock for $0 in ${FO_LOCKFILE}"
		if [ -f "$FO_LOCKFILE" ]; then
			kill -0 "$(cat $FO_LOCKFILE)" && another_instance
		fi
		echo $$ > "$FO_LOCKFILE"
	fi
}

fo_use_cache_file() {
	# Test if a cache file is valid, and delete it if invalid
	# Param: Cache File Suffix
	local usecache=false
	# read a cache file, if exist and recent
	if [ -f "$1" ]; then
		local lastentry=$(stat -c %Y "$1")
		local lastvalid=$(date -d "now -5 min" +%s)
		if [ $lastentry -gt $lastvalid ]; then
			usecache=true
		else
			rm "$1"
		fi
	elif [ -e "$1" ]; then
		rm "$1"
	fi
	if ! $usecache; then
		logger -t "udev" "[Rebuilding cache file]"
		# We should rebuild the cache: exit 1
		return 1
	else
		logger -t "udev" "[Using cache file]"
		# We can re-use the cache: exit 0
		return 0
	fi
}

fo_check_valid(){
	local v=$(dpkg-query -W lshw|cut -f2)
	dpkg --compare-versions ${v} ge 02.19 || readonly FO_LSHW_VALID=$?
  v=$(dpkg-query -W smartmontools|cut -f2)
	dpkg --compare-versions ${v} ge 7.2 || readonly FO_SMARTCTL_VALID=$?
  v=$(dpkg-query -W jq|cut -f2)
	dpkg --compare-versions ${v} ge 1.6 || readonly FO_JQ_VALID=$?
	v=$(dpkg-query -W storcli|cut -f2)
	dpkg --compare-versions ${v} ge 7.2508 || readonly FO_STORCLI_VALID=$?

}

fo_load_storcli(){
	if [ -z "$FO_JQ_VALID" ] && [ -z "$FO_STORCLI_VALID" ]; then
		fo_use_cache_file "${FO_CACHE_FILE_STORCLI}" || ${STORCLI} /call/eall/sall show all nolog J|jq -r '.["Controllers"][]["Response Data"]' > "${FO_CACHE_FILE_STORCLI}"
	fi
}

fo_close() {
  auxilium_close
	rm $FO_LOCKFILE 2> /dev/null || true
}

if [ -f "/etc/ferreus-orbit/ferreus-orbit.conf" ] ; then
	. "/etc/ferreus-orbit/ferreus-orbit.conf"
elif [ -f "etc/ferreus-orbit.conf" ] ; then
	. "etc/ferreus-orbit.conf"
else
	echo "Error: could not load ferreus-orbit.conf"
	exit 1
fi

readonly LSHW=/usr/bin/lshw
readonly FO_CACHE_FILE_LSHW=/tmp/${FO_SCRIPT}_cache_lshw
readonly FO_CACHE_FILE_STORCLI=/tmp/${FO_SCRIPT}_cache_storcli
readonly FO_CACHE_DISKS_INFOS=/tmp/${FO_SCRIPT}_disks_infos

readonly VAR_PATH='/var/cache/ferreus-orbit/'

if [ -x /usr/bin/storcli64 ] ; then
	readonly STORCLI=/usr/bin/storcli64
elif [ -x /usr/bin/storcli_x64 ] ; then
	readonly STORCLI=/usr/bin/storcli_x64
elif [ -x /usr/bin/storcli ] ; then
	readonly STORCLI=/usr/bin/storcli
fi

readonly FO_ETC_PLUGINS_PATH="/etc/ferreus-orbit/plugins"
if [ -d "/usr/share/ferreus-orbit/plugins" ]; then
	readonly FO_PLUGINS_PATH="/usr/share/ferreus-orbit/plugins"
elif [ -d "share/plugins" ] ; then
	readonly FO_PLUGINS_PATH="share/plugins"
else
	readonly FO_PLUGINS_PATH="../share/plugins"
fi

readonly FO_CHASSIS_MANUFACTURER=$(dmidecode -s system-manufacturer)
readonly FO_CHASSIS_MODEL=$(dmidecode -s system-product-name)

fo_check_valid
if [ -d /dev/disk/oci-sort ] ; then
	USE_FO_NAME=0
  readonly SORTED_PATH='/dev/disk/oci-sort'
elif [ -d /dev/disk/fo-sort ] ; then
	USE_FO_NAME=1
  readonly SORTED_PATH='/dev/disk/fo-sort'
else
	USE_FO_NAME=-1
  readonly SORTED_PATH='/dev'
fi
if [ -f "${FO_ETC_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}" ]; then
  . "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}"
elif [ -f "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}" ]; then
  . "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}"
elif [ -f "${FO_ETC_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic" ]; then
  . "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic"
elif [ -f "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic" ]; then
  . "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic"
else
  . "${FO_PLUGINS_PATH}/generic"
fi
